package personneDandD;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;

import javax.swing.JComponent;
import javax.swing.TransferHandler;

/*
 * Created on 4 avr. 2005 by rosmord
 *
 * This file is distributed according to the GNU LESSER PUBLIC LICENCE
 */

/**
 * Le PersonneTransferHandler est attach aux PersonnePanel, et gre le drag and
 * drop pour ceux-ci.
 * <p>
 * On pourrait utiliser ici le design pattern Singleton, pour que le mme
 * handler gre toutes les personnes.
 * 
 * @author rosmord
 * 
 *  
 */
public class PersonneTransferHandler extends TransferHandler {

    /**
     * Importation de donnes.
     * <p>
     * Quand la mthode est appele, c est la cible du transfert.
     */
    public boolean importData(JComponent c, Transferable t) {
        if (hasPersonneFlavor(t.getTransferDataFlavors())) {
            try {
                Personne p = (Personne) t.getTransferData(PersonneFlavorFactory
                        .getPersonneFlavor());
                ((PersonnePanel) c).setPersonne(p);
                return true;
            } catch (UnsupportedFlavorException ufe) {
                ufe.printStackTrace();
            } catch (IOException ioe) {
                ioe.printStackTrace();
            }
        }
        return false;
    }

    /**
     * Cration d'un Transferable encapsulant une personne.
     * <p>
     * c est la source du transfert.
     * 
     * @see javax.swing.TransferHandler#createTransferable(javax.swing.JComponent)
     */
    protected Transferable createTransferable(JComponent c) {
        PersonnePanel panel = (PersonnePanel) c;
        Personne p = panel.getPersonne();
        return new PersonneTransferable(p);
    }

    /**
     * Teste la prsence d'une personneFlavor dans la liste
     * 
     * @param flavors
     * @return
     */
    protected boolean hasPersonneFlavor(DataFlavor[] flavors) {
        // cas normalement impossible :
        if (PersonneFlavorFactory.getPersonneFlavor() == null) {
            return false;
        }

        for (int i = 0; i < flavors.length; i++) {
            if (PersonneFlavorFactory.getPersonneFlavor().equals(flavors[i])) {
                return true;
            }
        }
        return false;
    }

    /**
     * Teste si nous pouvons importer des donne proposes dans une liste de
     * flavor fixe.
     */
    public boolean canImport(JComponent c, DataFlavor[] flavors) {
        return hasPersonneFlavor(flavors);
    }

    /*
     * On ne gre que la copie, pas le mode MOVE (qui enlve l'objet de sa
     * source).
     * 
     * @see javax.swing.TransferHandler#getSourceActions(javax.swing.JComponent)
     */
    public int getSourceActions(JComponent c) {
        return COPY;
    }
    

}
