/*
 * This file is distributed according to the GNU Lesser Public Licence.
 * Created on 5 avr. 2005
 *
 */
package personneDandD;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ArrayList;


/**
 * Cette classe reprsente la donne qui est stocke en mmoire et utilise pour le drag.
 * Son travail est d'encapsuler une personne, et de transmettre l'objet correspondant et de le transmettre sous diffrentes formes,
 * selon les capacits de la cible.
 * 
 * <p> Si la cible accepte la flavor PersonneFlavor, alors, on lui envoie une Flavor ; sinon, si elle 
 * accepte du texte, on lui renvoie une String. 
 * @author Serge Rosmorduc
 */
public class PersonneTransferable implements Transferable {

   
	
    private static DataFlavor htmlFlavor8= new DataFlavor("text/html;charset=utf-8", null);
    //private static DataFlavor htmlFlavor= new DataFlavor("text/html;charset=unicode", null);
    private static DataFlavor htmlFlavor= new DataFlavor("text/html;class=java.lang.String", null);
    private static DataFlavor zorglubFlavor= new DataFlavor("text/html;class=java.lang.String", null);
    
    private static DataFlavor rtfFlavor= new DataFlavor("text/rtf", null);

    /**
     * 
     * @uml.property name="personne"
     * @uml.associationEnd multiplicity="(1 1)"
     */
    private Personne personne;

    
    /**
     * @param p
     */
    public PersonneTransferable(Personne p) {
       personne= p;
    }


    /**
     * La liste des flavor supportes, pour que les cibles potentielles sachent si elles 
     * peuvent recevoir la donne.
     */
    public DataFlavor[] getTransferDataFlavors() {
        System.out.println("appel getTransferDataFlavor");
        ArrayList l= new ArrayList(); 
        l.add(PersonneFlavorFactory.getPersonneFlavor());
        //l.add(new DataFlavor(DemoTransfert.class.getClass(), "image/png;class=java.io.InputStream"));
        l.add(rtfFlavor);
        l.add(zorglubFlavor);
        l.add(htmlFlavor);
        l.add(htmlFlavor8);
        l.add(DataFlavor.stringFlavor);
        return (DataFlavor[]) l.toArray(new DataFlavor[l.size()]);
        
    }

    /**
     * Dit si nous acceptons une flavor particulire.
     */
    public boolean isDataFlavorSupported(DataFlavor flavor) {
        System.out.println("appel isDataFlavorSupported");
        return PersonneFlavorFactory.getPersonneFlavor().equals(flavor) || DataFlavor.stringFlavor.equals(flavor);
    }

    /**
     * La rcupration des donnes proprement dite.
     * <p> Renvoie un objet selon la flavor demande (Personne ou String).
     */
    public Object getTransferData(DataFlavor flavor) throws UnsupportedFlavorException, IOException {
        System.out.println("flavor "+ flavor.toString());
        if (PersonneFlavorFactory.getPersonneFlavor().equals(flavor))
            return personne;
        else if (htmlFlavor.equals(flavor)) {
            System.out.println("trying to send html");
            String str= "<html><body>nom : <b>"+ personne.getNom()+ "</b> prenom :" + personne.getPrenom()+ "</body></html>";          
            return str;
        }
        else if (htmlFlavor8.equals(flavor)) {
        	System.out.println("trying to send html (doesn't work. Seems to work with unicode");
        	System.out.println(flavor.getParameter("charset"));
            java.io.InputStream io;
            byte [] buf;
            String str= "<html><body>nom : <b>"+ personne.getNom()+ "</b> prenom :" + personne.getPrenom()+ "</body></html>";
          
            ByteArrayOutputStream out= new ByteArrayOutputStream();
            OutputStreamWriter w= new OutputStreamWriter(out, "UTF-8");
            w.write(str);
            w.close();
            buf= out.toByteArray();
            io= new ByteArrayInputStream(buf);            
          return io;
        }
        else if (DataFlavor.stringFlavor.equals(flavor))
        {
            return personne.getNom().toUpperCase()+ " " + personne.getPrenom();
        }
        else throw new UnsupportedFlavorException(flavor);
    }

    
}
