/*
 * This file is distributed according to the GNU Lesser Public Licence.
 * Created on 6 avr. 2005
 *
 */
package targetted;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * Exemple de cible pour le drag and drop.
 * <p> Cette classe montre comment raliser un objet dont le comportement lors du drop dpend de la position de la souris.
 * Ce comportement n'est pas gr par les TransfertHandler.
 * 
 * @author Serge Rosmorduc
 */
public class TargetedDragAndDrop extends JPanel {
    
    
    String t[];
    /**
     * 
     */
    public TargetedDragAndDrop() {
        setBackground(Color.WHITE);
        t = new String[4];
        t[0]= "a";
        t[1]= "b";
        t[2]= "c";
        t[3]= "d";
        setPreferredSize(new Dimension(400,400));
        setDropTarget(new DropTarget(this, new MyDropTarget()));
        
    }

    /* (non-Javadoc)
     * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
     */
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        setFont(new Font("Dialog", Font.BOLD, 24));
        g.drawString(t[0],0,100);
        g.drawString(t[1],200,100);
        g.drawString(t[2],0,400);
        g.drawString(t[3],200,400);
    }
    
    public static void main(String[] args) {
        JFrame jf= new JFrame("test drop");
        jf.getContentPane().setLayout(new FlowLayout(FlowLayout.LEFT));
        jf.getContentPane().add(new TargetedDragAndDrop());
        jf.getContentPane().add(new JTextField(10));
        jf.pack();
        jf.addWindowListener(new WindowAdapter() {
           /* (non-Javadoc)
         * @see java.awt.event.WindowAdapter#windowClosing(java.awt.event.WindowEvent)
         */
        public void windowClosing(WindowEvent e) {
            System.exit(0);
        } 
        });
        jf.setVisible(true);
    }
 
    
    class MyDropTarget implements DropTargetListener {

        /* (non-Javadoc)
         * @see java.awt.dnd.DropTargetListener#dragEnter(java.awt.dnd.DropTargetDragEvent)
         */
        public void dragEnter(DropTargetDragEvent dtde) {
            System.out.println("enter");
            duringDrag(dtde);
        }

        /* (non-Javadoc)
         * @see java.awt.dnd.DropTargetListener#dragOver(java.awt.dnd.DropTargetDragEvent)
         */
        public void dragOver(DropTargetDragEvent dtde) {
            System.out.println("over");
            duringDrag(dtde);
        }

        /* (non-Javadoc)
         * @see java.awt.dnd.DropTargetListener#dropActionChanged(java.awt.dnd.DropTargetDragEvent)
         */
        public void dropActionChanged(DropTargetDragEvent dtde) {
            System.out.println("changed");
            duringDrag(dtde);
        }

        /* (non-Javadoc)
         * @see java.awt.dnd.DropTargetListener#dragExit(java.awt.dnd.DropTargetEvent)
         */
        public void dragExit(DropTargetEvent dte) {
            System.out.println("exit");
            
        }
        
        private void duringDrag(DropTargetDragEvent dtde) {
            System.out.println(dtde.getCurrentDataFlavorsAsList());
            if (! dtde.isDataFlavorSupported(DataFlavor.stringFlavor))
                dtde.rejectDrag();
        }

        /* (non-Javadoc)
         * @see java.awt.dnd.DropTargetListener#drop(java.awt.dnd.DropTargetDropEvent)
         */
        public void drop(DropTargetDropEvent dtde) {
            System.out.println("drop");
            // Accepter le drop :
            dtde.acceptDrop(dtde.getDropAction());
            // Le traiter
            Transferable tr= dtde.getTransferable();
            
            // Trouver la cible du drop
            Point p= dtde.getLocation();            
            int target= 0;
            if (p.x < getWidth() / 2) {
                // t[0] or t[2]
                if (p.y > getHeight()/2) {
                    target= 2;
                } else
                    target= 0;
            } else {
                if (p.y> getHeight() /2)
                    target= 3;
                else
                    target= 1;
            }
            
            // Essayer effectivement le drop.
            if (tr.isDataFlavorSupported(DataFlavor.getTextPlainUnicodeFlavor())) {
                try {             
                    t[target]= (String) tr.getTransferData(DataFlavor.stringFlavor);
                    dtde.dropComplete(true);
                } catch (UnsupportedFlavorException e) {
                  dtde.dropComplete(false);
                    e.printStackTrace();
                } catch (IOException e) {
                    dtde.dropComplete(false);
                    e.printStackTrace();
                }
            }      
            // On redessine.
            repaint();
        }
        
    }
}
